aboutsummaryrefslogtreecommitdiff
path: root/gdb/nat
diff options
context:
space:
mode:
authorAndrew Burgess <aburgess@redhat.com>2024-03-26 18:52:17 +0000
committerAndrew Burgess <aburgess@redhat.com>2024-03-26 18:52:17 +0000
commit69324a74e344e640e0c2d4e76e4e4dfd3368cd76 (patch)
treededb9438c712c5d366569c1601050d2a868cc8a6 /gdb/nat
parent57d8b51d901585a07524ece1188b1892b207e950 (diff)
downloadgdb-69324a74e344e640e0c2d4e76e4e4dfd3368cd76.zip
gdb-69324a74e344e640e0c2d4e76e4e4dfd3368cd76.tar.gz
gdb-69324a74e344e640e0c2d4e76e4e4dfd3368cd76.tar.bz2
Revert "gdb/gdbserver: share x86/linux tdesc caching"
This reverts commit 198ff6ff819c240545f9fc68b39636fd376d4ba9.
Diffstat (limited to 'gdb/nat')
-rw-r--r--gdb/nat/x86-linux-tdesc.c287
-rw-r--r--gdb/nat/x86-linux-tdesc.h56
2 files changed, 8 insertions, 335 deletions
diff --git a/gdb/nat/x86-linux-tdesc.c b/gdb/nat/x86-linux-tdesc.c
index c438dfa..be7014d 100644
--- a/gdb/nat/x86-linux-tdesc.c
+++ b/gdb/nat/x86-linux-tdesc.c
@@ -38,8 +38,6 @@
#include <sys/user.h>
#include <sys/user.h>
-#ifndef IN_PROCESS_AGENT
-
/* See nat/x86-linux-tdesc.h. */
const target_desc *
@@ -124,288 +122,3 @@ x86_linux_tdesc_for_tid (int tid, enum tribool *have_ptrace_getregset,
gdb_assert_not_reached ("failed to return tdesc");
}
-
-#endif /* !IN_PROCESS_AGENT */
-
-
-
-/* A structure used to describe a single cpu feature that might, or might
- not, be checked for when creating a target description for one of i386,
- amd64, or x32. */
-
-struct x86_tdesc_feature {
- /* The cpu feature mask. This is a mask against an xcr0 value. */
- uint64_t feature;
-
- /* Is this feature checked when creating an i386 target description. */
- bool is_i386;
-
- /* Is this feature checked when creating an amd64 target description. */
- bool is_amd64;
-
- /* Is this feature checked when creating an x32 target description. */
- bool is_x32;
-};
-
-/* A constant table that describes all of the cpu features that are
- checked when building a target description for i386, amd64, or x32. */
-
-static constexpr x86_tdesc_feature x86_linux_all_tdesc_features[] = {
- /* Feature, i386, amd64, x32. */
- { X86_XSTATE_PKRU, true, true, true },
- { X86_XSTATE_AVX512, true, true, true },
- { X86_XSTATE_AVX, true, true, true },
- { X86_XSTATE_MPX, true, true, false },
- { X86_XSTATE_SSE, true, false, false },
- { X86_XSTATE_X87, true, false, false }
-};
-
-/* Return a compile time constant which is a mask of all the cpu features
- that are checked for when building an i386 target description. */
-
-static constexpr uint64_t
-x86_linux_i386_tdesc_feature_mask ()
-{
- uint64_t mask = 0;
-
- for (const auto &entry : x86_linux_all_tdesc_features)
- if (entry.is_i386)
- mask |= entry.feature;
-
- return mask;
-}
-
-/* Return a compile time constant which is a mask of all the cpu features
- that are checked for when building an amd64 target description. */
-
-static constexpr uint64_t
-x86_linux_amd64_tdesc_feature_mask ()
-{
- uint64_t mask = 0;
-
- for (const auto &entry : x86_linux_all_tdesc_features)
- if (entry.is_amd64)
- mask |= entry.feature;
-
- return mask;
-}
-
-/* Return a compile time constant which is a mask of all the cpu features
- that are checked for when building an x32 target description. */
-
-static constexpr uint64_t
-x86_linux_x32_tdesc_feature_mask ()
-{
- uint64_t mask = 0;
-
- for (const auto &entry : x86_linux_all_tdesc_features)
- if (entry.is_x32)
- mask |= entry.feature;
-
- return mask;
-}
-
-/* Return a compile time constant which is a count of the number of cpu
- features that are checked for when building an i386 target description. */
-
-static constexpr int
-x86_linux_i386_tdesc_count ()
-{
- uint64_t count = 0;
-
- for (const auto &entry : x86_linux_all_tdesc_features)
- if (entry.is_i386)
- ++count;
-
- gdb_assert (count > 0);
-
- return (1 << count);
-}
-
-/* Return a compile time constant which is a count of the number of cpu
- features that are checked for when building an amd64 target description. */
-
-static constexpr int
-x86_linux_amd64_tdesc_count ()
-{
- uint64_t count = 0;
-
- for (const auto &entry : x86_linux_all_tdesc_features)
- if (entry.is_amd64)
- ++count;
-
- gdb_assert (count > 0);
-
- return (1 << count);
-}
-
-/* Return a compile time constant which is a count of the number of cpu
- features that are checked for when building an x32 target description. */
-
-static constexpr int
-x86_linux_x32_tdesc_count ()
-{
- uint64_t count = 0;
-
- for (const auto &entry : x86_linux_all_tdesc_features)
- if (entry.is_x32)
- ++count;
-
- gdb_assert (count > 0);
-
- return (1 << count);
-}
-
-#ifdef IN_PROCESS_AGENT
-
-/* See linux-x86-tdesc.h. */
-
-int
-x86_linux_amd64_ipa_tdesc_count ()
-{
- return x86_linux_amd64_tdesc_count ();
-}
-
-/* See linux-x86-tdesc.h. */
-
-int
-x86_linux_x32_ipa_tdesc_count ()
-{
- return x86_linux_x32_tdesc_count ();
-}
-
-/* See linux-x86-tdesc.h. */
-
-int
-x86_linux_i386_ipa_tdesc_count ()
-{
- return x86_linux_i386_tdesc_count ();
-}
-
-#endif /* IN_PROCESS_AGENT */
-
-/* Convert an xcr0 value into an integer. The integer will be passed to
- the in-process-agent where it will then be passed to
- x86_linux_tdesc_idx_to_xcr0 to get back the xcr0 value. */
-
-int
-x86_linux_xcr0_to_tdesc_idx (uint64_t xcr0)
-{
- /* The following table shows which features are checked for when creating
- the target descriptions (see nat/x86-linux-tdesc.c), the feature order
- represents the bit order within the generated index number.
-
- i386 | x87 sse mpx avx avx512 pkru
- amd64 | mpx avx avx512 pkru
- i32 | avx avx512 pkru
-
- The features are ordered so that for each mode (i386, amd64, i32) the
- generated index will form a continuous range. */
-
- int idx = 0;
-
- for (int i = 0; i < ARRAY_SIZE (x86_linux_all_tdesc_features); ++i)
- {
- if ((xcr0 & x86_linux_all_tdesc_features[i].feature) != 0)
- idx |= (1 << i);
- }
-
- return idx;
-}
-
-
-#ifdef IN_PROCESS_AGENT
-
-/* Convert an index number (as returned from x86_linux_xcr0_to_tdesc_idx)
- into an xcr0 value which can then be used to create a target
- description. */
-
-uint64_t
-x86_linux_tdesc_idx_to_xcr0 (int idx)
-{
- uint64_t xcr0 = 0;
-
- for (int i = 0; i < ARRAY_SIZE (x86_linux_all_tdesc_features); ++i)
- {
- if ((idx & (1 << i)) != 0)
- xcr0 |= x86_linux_all_tdesc_features[i].feature;
- }
-
- return xcr0;
-}
-
-#endif /* IN_PROCESS_AGENT */
-
-#if defined __i386__ || !defined IN_PROCESS_AGENT
-
-/* A cache of all possible i386 target descriptions. */
-
-static struct target_desc *i386_tdescs[x86_linux_i386_tdesc_count ()] = { };
-
-/* See nat/x86-linux-tdesc.h. */
-
-const struct target_desc *
-i386_linux_read_description (uint64_t xcr0)
-{
- xcr0 &= x86_linux_i386_tdesc_feature_mask ();
- int idx = x86_linux_xcr0_to_tdesc_idx (xcr0);
-
- gdb_assert (idx >= 0 && idx < x86_linux_i386_tdesc_count ());
-
- target_desc **tdesc = &i386_tdescs[idx];
-
- if (*tdesc == nullptr)
- {
- *tdesc = i386_create_target_description (xcr0, true, false);
-
- x86_linux_post_init_tdesc (*tdesc, false);
- }
-
- return *tdesc;
-}
-#endif
-
-#ifdef __x86_64__
-
-/* A cache of all possible amd64 target descriptions. */
-
-static target_desc *amd64_tdescs[x86_linux_amd64_tdesc_count ()] = { };
-
-/* A cache of all possible x32 target descriptions. */
-
-static target_desc *x32_tdescs[x86_linux_x32_tdesc_count ()] = { };
-
-/* See nat/x86-linux-tdesc.h. */
-
-const struct target_desc *
-amd64_linux_read_description (uint64_t xcr0, bool is_x32)
-{
- if (is_x32)
- xcr0 &= x86_linux_x32_tdesc_feature_mask ();
- else
- xcr0 &= x86_linux_amd64_tdesc_feature_mask ();
-
- int idx = x86_linux_xcr0_to_tdesc_idx (xcr0);
-
- if (is_x32)
- gdb_assert (idx >= 0 && idx < x86_linux_x32_tdesc_count ());
- else
- gdb_assert (idx >= 0 && idx < x86_linux_amd64_tdesc_count ());
-
- target_desc **tdesc = nullptr;
-
- if (is_x32)
- tdesc = &x32_tdescs[idx];
- else
- tdesc = &amd64_tdescs[idx];
-
- if (*tdesc == nullptr)
- {
- *tdesc = amd64_create_target_description (xcr0, is_x32, true, true);
-
- x86_linux_post_init_tdesc (*tdesc, true);
- }
- return *tdesc;
-}
-
-#endif
diff --git a/gdb/nat/x86-linux-tdesc.h b/gdb/nat/x86-linux-tdesc.h
index 648fe0e..3727a8b 100644
--- a/gdb/nat/x86-linux-tdesc.h
+++ b/gdb/nat/x86-linux-tdesc.h
@@ -24,8 +24,6 @@
struct target_desc;
-#ifndef IN_PROCESS_AGENT
-
/* Return the target description for Linux thread TID.
When *HAVE_PTRACE_GETREGSET is TRIBOOL_UNKNOWN then the current value of
@@ -59,57 +57,19 @@ x86_linux_tdesc_for_tid (int tid, enum tribool *have_ptrace_getregset,
gdb::function_view<void (uint64_t)> xcr0_init_cb,
const char *error_msg, uint64_t *xcr0_storage);
-#endif /* !IN_PROCESS_AGENT */
-
#ifdef __x86_64__
-/* Return the AMD64 target descriptions corresponding to XCR0 and IS_X32. */
+/* Return the right amd64-linux target descriptions according to
+ XCR0_FEATURES_BIT and IS_X32. This is implemented separately in both
+ GDB and gdbserver. */
-extern const target_desc *amd64_linux_read_description (uint64_t xcr0,
- bool is_x32);
+extern const target_desc *amd64_linux_read_description
+ (uint64_t xcr0_features_bit, bool is_x32);
-#endif /* __x86_64__ */
-
-/* Return the i386 target description corresponding to XCR0. */
+#endif
+/* Return the target description according to XCR0. This is implemented
+ separately in both GDB and gdbserver. */
extern const struct target_desc *i386_linux_read_description (uint64_t xcr0);
-/* This function is called from amd64_linux_read_description and
- i386_linux_read_description after a new target description has been
- created, TDESC is the new target description, IS_64BIT will be true
- when called from amd64_linux_read_description, otherwise IS_64BIT will
- be false. If the *_linux_read_description functions found a cached
- target description then this function will not be called.
-
- Both GDB and gdbserver have their own implementations of this
- function. */
-
-extern void x86_linux_post_init_tdesc (target_desc *tdesc, bool is_64bit);
-
-/* Convert an xcr0 value into an integer. The integer will be passed to
- the in-process-agent where it will then be passed to
- x86_linux_tdesc_idx_to_xcr0 to get back the xcr0 value. */
-
-extern int x86_linux_xcr0_to_tdesc_idx (uint64_t xcr0);
-
-
-#ifdef IN_PROCESS_AGENT
-
-/* Convert an index number (as returned from x86_linux_xcr0_to_tdesc_idx)
- into an xcr0 value which can then be used to create a target
- description. */
-
-extern uint64_t x86_linux_tdesc_idx_to_xcr0 (int idx);
-
-/* Within the in-process-agent we need to pre-initialise all of the target
- descriptions, to do this we need to know how many target descriptions
- there are for each different target type. These functions return the
- target description count for the relevant target. */
-
-extern int x86_linux_amd64_ipa_tdesc_count ();
-extern int x86_linux_x32_ipa_tdesc_count ();
-extern int x86_linux_i386_ipa_tdesc_count ();
-
-#endif /* IN_PROCESS_AGENT */
-
#endif /* NAT_X86_LINUX_TDESC_H */